% Copyright (C) 2001, James B. Rawlings and John G. Ekerdt
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License as
% published by the Free Software Foundation; either version 2, or (at
% your option) any later version.
%
% This program is distributed in the hope that it will be useful, but
% WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
% General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; see the file COPYING.  If not, write to
% the Free Software Foundation, 59 Temple Place - Suite 330, Boston,
% MA 02111-1307, USA.

global  npts A Aint Bint Rint  caf cbf ccf ...
        k10 E1 k20 E2 Ka0 Ea Kc0 Ec Da Db Dc T ...
        Ra Rb Rc Da Db Dc kma kmb kmc dcadr dcbdr dccdr
%
% Compute the pellet concentration profiles.
%
% Method can address:
%
%           multi-component, multi-reaction, arbitrary kinetics
%           reaction-diffusion problem in pellet, accurate profiles
%           flux and/or value boundary conditions at pellet exterior
%           energy balance in pellet and fluid
%           pressure drop and equation of state for fluid
%           sphere, semi-infinite cylinder or slab geometries for pellet
%           axial dispersion in the bed
%           transient problem
%
% Method does not address:
%
%           radial profiles in bed
%           non-uniform pellet exterior
%
% 5/07/02
%
% added log transformation; z=log(c)
% 12/ 13/03
%
% jbr
%
%
% units: mol, cm, sec, K
%
% Simulate pellet with parameters from Co, Cavendish, Hegedus.
%
% C0     + 1/2 O_2 -->   CO_2
%
% C_3H_6 + 9/2 O_2 --> 3 CO_2 + 3 H_2O
%
% A=CO  B=O_2  C=C_3H_6  D=CO_2  E=H_2O
%
% r_1 = k_1 c_A c_B / (1+K_A c_A + K_C c_C)^2
% r_2 = k_2 c_C c_B / (1+K_A c_A + K_C c_C)^2

Rg  = 8.314;  % J/K mol
P   = 1.013e5;  % N/m^2
T   = 550; % K
cf  = P/(Rg*T)*1e-6; % mol/cm^3
Rp  = 0.175; % cm radius of catalyst particle

caf = cf*0.02;
cbf = cf*0.03;
ccf = cf*5e-4;

k10  = 6.802e16*2.6e6*80/100*0.05/100;  %mol/cm^3 s
k20  = 1.416e18*2.6e6*80/100*0.05/100;  %mol/cm^3 s
E1   = 13108; %K
E2   = 15109; %K
Ka0  = 8.099e6; % cm^3/mol
Kc0  = 2.579e8; % cm^3/mol
Ea   = -409; %K
Ec   = 191; %K
Da   = 0.0487; % cm^2/s
Db   = 0.0469; % cm^2/s
Dc   = 0.0487; % cm^2/s
kma  = 0.4*9.76;   % cm/s
kmb  = 0.4*10.18;  % cm/s
kmc  = 0.4*9.76;   % cm/s
%
% global collocation
%
npts = 80;
[R A B Q] = colloc(npts-2, 'left', 'right');
R = R*Rp;
A = A/Rp;
B = B/(Rp*Rp);
Q = Q*Rp;
Aint = A(2:npts-1,:);
Bint = B(2:npts-1,:);
Rint = R(2:npts-1);
%
%
%
% find the pellet profile at the fluid conditions
%
zain = log(1e-9*caf); zaout = log(0.75*caf);
za0 = zain + R/Rp*(zaout-zain);
zbin = log(0.75*cbf); zbout = log(cbf);
zb0 = zbin + R/Rp*(zbout-zbin);
zcin = log(1e-6*ccf); zcout = log(ccf);
zc0 = zcin + R/Rp*(zcout-zcin);

z0 = [za0;zb0;zc0];
opts = optimset ('MaxFunEvals', 2000*numel (z0), ...
                 'MaxIter', 500*numel (z0), ...
		 'TolX', 1e-10, 'TolFun', 1e-15);
[z,fval,info] = fsolve('pellet',z0,opts);

za = z(1:npts);
zb = z(npts+1:2*npts);
zc = z(2*npts+1:3*npts);

ca = exp(za);
cb = exp(zb);
cc = exp(zc);
%
% compute other products concentrations
%
kmd = kma;
kme = kmb;
Dd  = Da;
De  = Db;
cdf = 0;
cef = 0;
dcddr = (-Da*dcadr(npts)-3*Dc*dccdr(npts)) /Dd ;
dcedr = (-3*Dc*dccdr(npts)) /De ;
cdR   = cdf - Dd*dcddr/kmd;
ceR   = cef - De*dcedr/kme;
concd = ( Da*(ca(npts)-ca) + 3*Dc*(cc(npts)-cc) )/Dd + cdR;
ce    = ( 3*Dc*(cc(npts)-cc) )/De + ceR;

table = [R ca cb cc concd ce dcadr dcbdr dccdr];

table_2 = [Rp caf cbf ccf cdf cef];

save multi_log.dat table table_2;

if (~ strcmp (getenv ('OMIT_PLOTS'), 'true')) % PLOTTING
subplot (3, 1, 1);
plot (table(:,1), table(:,2:4), table_2(:,1), table_2(:,2:4));
% TITLE

subplot (3, 1, 2);
semilogy (table(:,1), table(:,2:4), table_2(:,1), table_2(:,2:4));
% TITLE

subplot (3, 1, 3);
plot (table(:,1), table(:,5:6), table_2(:,1), table_2(:,5:6));
% TITLE
end % PLOTTING